package uddi_org.service;

import java.io.IOException;
import java.net.MalformedURLException;
import java.util.Vector;
import java.util.logging.FileHandler;
import java.util.logging.Formatter;
import java.util.logging.Level;
import java.util.logging.Logger;
import java.util.logging.SimpleFormatter;

import org.uddi4j.UDDIException;
import org.uddi4j.client.UDDIProxy;
import org.uddi4j.datatype.binding.AccessPoint;
import org.uddi4j.datatype.binding.BindingTemplate;
import org.uddi4j.datatype.binding.BindingTemplates;
import org.uddi4j.datatype.binding.TModelInstanceInfo;
import org.uddi4j.datatype.business.BusinessEntity;
import org.uddi4j.datatype.service.BusinessService;
import org.uddi4j.datatype.service.BusinessServices;
import org.uddi4j.datatype.tmodel.TModel;
import org.uddi4j.response.BindingDetail;
import org.uddi4j.response.BusinessDetail;
import org.uddi4j.response.DispositionReport;
import org.uddi4j.response.ErrInfo;
import org.uddi4j.response.Result;
import org.uddi4j.response.ServiceDetail;
import org.uddi4j.response.TModelDetail;
import org.uddi4j.response.TModelInfo;
import org.uddi4j.transport.TransportException;
import org.uddi4j.util.KeyedReference;

import utils.Util_TypeMapping;

/**
 * @author faith  
 * 
 * Mar 14, 2007  2:23:04 PM
 */
public class AuditionManager {

	//BT state
	private Vector<BindingTemplate> bindingDetailVector;
	private boolean allBindingTestok;
	private DispositionReport bindingDispositionReport;

	//SERVICE state
	private Vector<BusinessService> serviceDetailVector;
	private boolean allServiceTestok;
	private DispositionReport serviceDispositionReport;

	//BUSINESS ENTITY state
	private Vector<BusinessEntity> businessDetailVector;
	private boolean allBusinessTestok;
	private DispositionReport businessDispositionReport;

	//UDDI4J UDDI Proxy
	private UDDIProxy proxy;

	//shared state
	private PendingTModel pendingt;
	private PendingBindingTemplate pendingbt;

	//the code 70000 can be adopted, it doesn't appear in the UDDI V2 spec
	private static String errno_testFaild = "70000";

	//errinfo content: is the key, in case of succeed or update, errno_testFaild_description string otherwise
	private static String errno_testFaild_description = "Audition process failed";

	private static String errno_success = "0";

	//errcode
	private static String E_testFaild = "E_testFaild";

	//logger vars
	protected static Logger logger = Logger.getLogger("AuditionManager");
	protected static FileHandler fh;


	/**
	 * default constructor
	 */
	public AuditionManager()
	{
		//initialize UDDI4J proxy
		this.proxy = new UDDIProxy();
		try {
			proxy.setInquiryURL(Constants.INQUIRY_URL_STRING);
			proxy.setPublishURL(Constants.PUBLISH_URL_STRING);
		} catch (MalformedURLException e) {return;}


		//BINDING TEMPLATE vars
		//initialize binding DispositionReport dispositionReport
		this.bindingDispositionReport = new DispositionReport();
		this.bindingDispositionReport.setGeneric(DispositionReport.GENERIC);
		this.bindingDispositionReport.setOperator(Constants.OPERATOR);
		this.bindingDispositionReport.setTruncated(false);

		//initialize Vector<BindingTemplate> bindingDetailVector
		this.bindingDetailVector = new Vector<BindingTemplate>();

		//initialize allTestok
		this.allBindingTestok = false;

		//BUSINESS SERVICE vars 		
		//initialize serviceDispositionReport dispositionReport
		this.serviceDispositionReport = new DispositionReport();
		this.serviceDispositionReport.setGeneric(DispositionReport.GENERIC);
		this.serviceDispositionReport.setOperator(Constants.OPERATOR);
		this.serviceDispositionReport.setTruncated(false);

		// service Detail vector
		this.serviceDetailVector = new Vector<BusinessService>();

		// service all test ok bool var
		this.allServiceTestok = false;

		//BUSIENSS ENTITY vars
		//initialize serviceDispositionReport dispositionReport
		this.businessDispositionReport = new DispositionReport();
		this.businessDispositionReport.setGeneric(DispositionReport.GENERIC);
		this.businessDispositionReport.setOperator(Constants.OPERATOR);
		this.businessDispositionReport.setTruncated(false);

		// service Detail vector
		this.businessDetailVector = new Vector<BusinessEntity>();

		// service all test ok bool var
		this.allBusinessTestok = false;

		//init logger
		try {
			if (fh == null) {
				fh = new FileHandler("/home/faith/workspace/UDDIProxyWS/AuditionManager.log");
				Formatter fmt = new SimpleFormatter();
				fh.setFormatter(fmt);
				logger.addHandler(fh);
				logger.setLevel(Level.ALL);}
		}catch(IOException e) {e.printStackTrace();}

		logger.info("AuditionManager init");
	}

	/*
	 * This testing phase is to be done before actually save the bindingTemplate in the UDDI registy,
	 * all needed info are taken from vector templates
	 * 
	 * Repeat for each BindingTemplate in templates
	 * 
	 * this api is invoked to save a BRAND NEW bindingTemplate or to UPDATE EXISTING bindintTemplate
	 * need to distingush the two cases 
	 *
	 * - the servieKey is a known value
	 * - *the wsdl:service name is indicated inside uddi structure: 
	 *    businessService/categoryBag/keyedReference/@keyName="service local name"
	 * 
	 * 1. retrieve info for testing purpose
	 *    1.1 get the accessPoint(s) (only ap with URLType = "http", "https") 
	 *    		see class uddi_org.api_v2.URLType
	 *    		see WS-I Basic Profile	
	 *    
	 *    1.2 wsdl url, service*, port, portType and binding inside that wsdl, of a given bt: go through its tModels 
	 *    find the tModel-portType (tmodel/categoryBag/keyedRefernce/@keyValue="portType") 
	 *    								and get the wsdl url in /tmodel/overviewURL/overviewDoc
	 *    find the tModel-binding (tmodel/categoryBag/keyedRefernce/@keyValue="binding") and get the wsdl url
	 *   								 and get the wsdl url in /tmodel/overviewURL/overviewDoc
	 *    
	 *    uddi:tmodel/name = wsdl:portType/@name, wsdl:binding/@name 
	 *    
	 *    riconoscere i tModel che sono correlati a wsdl: tmodel/categoryBag/keyedRefernce/@keyValue="wsdlSpec"
	 *    cfr WS-I Basic Profile 
	 *    
	 *    forse basta solo trovare il wsdl:service e wsdl:port name e da port risalire a binding - portType
	 *    in questo caso faccio un find_tmodel invece di scorrere tutti i tmodel
	 *    
	 *    potrebbero essere:
	 *    - lo stesso wsdl
	 *    - due o più wsdl diversi (astratto-concreto, import)
	 *    in questo caso wsdl2java risolverà le import collegandosi all'indirizzo remoto
	 *    
	 *    a me ne basta uno
	 *    
	 * 2. marking the service to be saved as pending
	 * 3. start the test engine, only if at least one valid accessPoint with one wsdl is provided
	 * 	  boolean testingResult = startTestEngine(...);
	 * 
	 * 4. test engine starts making invocation on 
	 * 
	 * 5. if (testingResult == true) 
	 * 			remove pending entry in serviceState
	 * 			!!-- make tmodel visible
	 * 			go on with registration process
	 * 			return the BindingDetail structure
	 *    else
	 *    		remove pending entry in serviceState
	 *    		!!-- delete tmodel, empty service can remain
	 *    		- throw DispositionReportException 
	 *    		or
	 *    		- return an empty BusinessDetail structure (with 0 bindingTemplate)
	 *    		in this case the client wont be able to know what's wrong 
	 * 
	 */

	public void bindingTemplateTesting(String authInfo, Vector templates)
	throws DispositionReportException, TransportException
	{
		try {
			//variable to check the vector size
			int num_template = templates.size();

			//variable to check the numer of succeeded test
			int succeededTest = 0;

			//nothing to do
			if (num_template == 0)
			{
				logger.info("No binding template to save");
				return;
			}

			//BindingTemplate Under Test
			BindingTemplate templateut = null;

			//vector, container for the binding template to save
			Vector<BindingTemplate> templateContainer= new Vector<BindingTemplate>();

			//to keep the result of save_binding 
			BindingDetail bindingDetail = null;

			//testing info: keeps info to invoke the testing engine
			AmbitionPars testinginfo = null;

			//
			Result result = null;
			ErrInfo errinfo = null;
			Vector<Result> resultVector = new Vector<Result>();

			//
			boolean testingResult = true;

			for(int i = 0; i<num_template; i++)
			{
				//binding template under test
				templateut = (BindingTemplate)templates.elementAt(i);

				//testinginfo to fill 
				testinginfo = new AmbitionPars();

				//the binding template key
				String bindingKey = templateut.getBindingKey();

				//the service key of the business service this bt belongs to
				String serviceKey = templateut.getServiceKey();

				ServiceDetail serviceDetail = null;
				BusinessService businessService = null;
				Vector keyedReference = null;

				//get the service name this bindingTemplate belongs to
				serviceDetail = this.proxy.get_serviceDetail(serviceKey);
				businessService = (BusinessService)(serviceDetail.getBusinessServiceVector().firstElement());
				keyedReference = businessService.getCategoryBag().getKeyedReferenceVector();

				//PARAMETER 3: the service QName 

				//look for canonical tmodel XML local name
				testinginfo.setServiceLocalName(getServiceLocalName(keyedReference));

				//look for canonical tmodel XML namespace
				testinginfo.setServiceNamespace(getServiceNamespace(keyedReference));

				//get the accessPoint
				testinginfo.setAccessPoint(getAccessPoint(templateut.getAccessPoint()));

				//build the service id
				ServiceId serviceId = new ServiceId(templateut.getAccessPoint());

				//mark the bindingTemplate as pending (just before the testing engine starts)	
				/*
				 * Why do I need this?
				 * When the testing engine starts it behaves like the service.
				 * Probaily, the engine will ask the registry for other web services
				 * since it is not the service itself a *real* service, but its interactions are 
				 * guided by the engine the proxy registry can't return real registered services.
				 */
				this.pendingbt.addBinding(serviceId);

				logger.info("Pending BEFORE engine starts: "+this.pendingbt.toString()+ " "+ this.pendingt.toString());

				//set the service id
				testinginfo.setServiceId(serviceId);

				String tmodelkey = null;
				String tmodelbindingkey = null;
				String tmodelservicekey = null;
				Vector tmodeldetails = templateut.getTModelInstanceDetails().getTModelInstanceInfoVector();
				TModelInstanceInfo instanceinfo = null;
				TModelDetail get = null;
				TModel getmodel = null;
				int j = 0;

				/*
				 * the URI of the WSDL location can be in one of the three tModel
				 * depending if the WSDL origination this registration is
				 * - complete WSDL
				 * - not-complete WSDL
				 */
				//String servicetmodel_URI = null;

				/*
				 * no assumption on the order of elements in tModelInstanceDetails is made
				 * portType tModel AND
				 * binding tModel AND
				 * service tModel AND
				 * SSM tModel MUST ALL BE PRESENT
				 * and MUST be already been registered in the registry
				 */

				//going through the tModelInstanceDetails 
				while ((j < tmodeldetails.size()))
				{
					instanceinfo = (TModelInstanceInfo) tmodeldetails.elementAt(j);
					j++;

					tmodelkey = instanceinfo.getTModelKey();
					//logger.info(tmodelkey);


					//get the complete tModel structure from this reference
					get = this.proxy.get_tModelDetail(tmodelkey);
					getmodel = (TModel) (get.getTModelVector().firstElement());

					//find out what kind of tModel this instanceInfo points to
					int type = switchtModelType(getmodel.getCategoryBag().getKeyedReferenceVector());

					switch(type)
					{
					//is the portType tModel
					case 1: 
					{
						testinginfo.setPortTypeName(getmodel.getNameString()); 
						break;
					}

					//is the binding tModel: in this case get the port local name
					case 2: 
					{
						tmodelbindingkey = tmodelkey;
						//testinginfo.setWsdlURI((getmodel.getOverviewDoc().getOverviewURLString()));
						testinginfo.setBindingName(getmodel.getNameString());

						//PAREMETER 4: port local name, the namespace is the same as the service 
						testinginfo.setPortLocalName(instanceinfo.getInstanceDetails().getInstanceParms().getText());
						break;
					}

					//is the service tModel
					case 3:
					{
						//PARAMETER 2: WSDL URI, the location of the WSDL document containing the wsdl:service element
						tmodelservicekey = tmodelkey;
						//servicetmodel_URI = getmodel.getOverviewDoc().getOverviewURLString();
						testinginfo.setWsdlURI((getmodel.getOverviewDoc().getOverviewURLString()));
						break;
					}

					//is the SSM tModel
					case 4:
					{
						//PARAMETER 1: SSM URI, the location of the SSM instance specification
						testinginfo.setSsmURI(getmodel.getOverviewDoc().getOverviewURLString());
						break;
					}
					}

				}

				/*
				 * External wsdl documents management is explained in the report for this project
				 * see chapter 4 for more details.
				 * 
				 * External wsdl documents can't be registerd in the audition
				 * enabled registry. Given this assumption, no check is done here.
				 * 
				 *	
				 * The accesspoint is a WSDL implementation document
				 * see uddi-spec-tc-tn-wsdl-v202
				 * 
				 * the value of accesspoint is a URL of the WSDL Implementaion Document
				 * in this case the ap is not mapped in UDDI: no representation for the ap is in the 
				 * registry. It is not possible to mark the service as pending
				 * Possibilities:
				 * 1. do not test the service, the service is registered as is
				 * 2. get the WSDL URL, get the WSDL document, get the ap : it could be not always possible   
				 *      
				 * In this implementation alternative 1 is considered assuming NO ONE REGISTER a 
				 * WSDL implementation document in this registry
				 * 
				 * Simply ignore this:
				 * if (externalWSDL == true)
				 * 
				 */ 

				logger.info("Extracted testing info: "+testinginfo.toString());

				/*
				 * TODO: AUDITION CALL
				 */

				//TestEngine engine = new TestEngine(testinginfo);
				//how long will it take?
				//testingResult = engine.startTesting(testinginfo);


				//create a Result object and ErrInfo for the binding template under test
				result = new Result();
				result.setKeyType(uddi_org.api_v2.KeyType.BINDING_KEY.toString());
				errinfo = new ErrInfo();

				//clean the vector container: to be sure only one binding at a time will be saved
				templateContainer.removeAllElements();

				//test ok
				if (testingResult == true)
				{				
					//add the template to save to the vector container
					templateContainer.add(templateut);

					//try to save this in the registry
					bindingDetail = this.proxy.save_binding(authInfo, templateContainer);


					succeededTest++;

					//save the bindingTemplate contained in the bindingDetail just saved
					this.bindingDetailVector.add((BindingTemplate)bindingDetail.getBindingTemplateVector().elementAt(0));

					/*
					 * if this is an update, the businesskey is the same as before
					 * else - this is a create -  save the just generated key
					 */
					bindingKey = ((BindingTemplate) (bindingDetail.getBindingTemplateVector().elementAt(0))).getBindingKey();
					//System.out.println(this.pendingToString());

					//mark as not pending related tmodelbinding 
					this.pendingt.removeTModel(new TModelKey(tmodelbindingkey));

					//mark as not pending related tmodelservice
					this.pendingt.removeTModel(new TModelKey(tmodelservicekey));

					/* set infos in Result object:
					 * - errno attribute: 0
					 * - errinfo content: the bindingkey
					 * - errcode attribute: E_success
					 */
					result.setErrno(errno_success);
					errinfo.setText(bindingKey);
					errinfo.setErrCode(DispositionReport.E_success);
				}

				//test failed
				/*
				 * - binding tmodel and service tmodel associated to the faild service
				 * are deleted; 
				 * - regitration is not executed
				 * - ad adeguate result element is generated
				 */
				else 
				{
					//delete related tmodelbinding 
					this.proxy.delete_tModel(authInfo, tmodelbindingkey);

					//delete related tmodel service
					this.proxy.delete_tModel(authInfo, tmodelservicekey);

					logger.info("else, pending failed: "+this.pendingbt.toString()+ " "+ this.pendingt.toString());

					/* set infos in Result object:
					 * - errno attribute: XXXX
					 * - errinfo content: empty
					 * - errcode attribute: E_testFaild
					 */
					result.setErrno(errno_testFaild);
					errinfo.setErrCode(AuditionManager.E_testFaild);
					errinfo.setText(errno_testFaild_description);

					/*
					 * do not TODO
					 * delete related business service: maybe the service contain other binding template
					 * 
					 */
				}

				//in both cases the bindingTemplate is no more pending
				//logger.info("remove pending serv: "+ testinginfo.getServiceId());
				this.pendingbt.removeBinding(testinginfo.getServiceId());

				logger.info("Pending AFTER engine starts: "+this.pendingbt.toString()+ " "+ this.pendingt.toString());

				/*
				 * set errinfo in Result, add the Result object to vector
				 * Results objects in resultVector are in the same order as binding templates has
				 * been processed 
				 */
				result.setErrInfo(errinfo);
				resultVector.add(result);
			}

			this.bindingDispositionReport.setResultVector(resultVector);

			if (succeededTest == num_template) 
			{
				logger.info("all bindingTemplate tests ok");
				this.allBindingTestok = true;
			}
			else
			{
				logger.info("all bindingTemplatetests failed");
				this.allBindingTestok = false;
			}
		}	
		catch (UDDIException e) {
			//throw a new DispositionReportException
			DispositionReportException dre = Util_TypeMapping.checkUDDIException(e);
			e.printStackTrace();
			throw dre;	} 
		catch(Exception e ) {
			logger.info("Generic Exception in Audition Manager, testingBindingTemplate");
			logger.info(e.getMessage());
			this.pendingbt.clear();
			this.pendingt.clear();
		}
	}


	/**
	 * @param authInfo
	 * @param services
	 * @throws DispositionReportException
	 * 
	 * 
	 * for each service to save:
	 * - if all included bt are test ok, the service is registered 
	 *   (success Report object and ServiceDetail object are created)  
	 * - if one of them fails, a new fail Report entry in DispositionReport,
	 *   included in DispositinReportException, is created
	 *  
	 * 
	 * a ServiceDetail structure is returned to caller only if all services have been registered
	 */
	public void businessServiceTesting(String authInfo, Vector services) 
	throws DispositionReportException, TransportException 
	{
		//service under test
		BusinessService serviceut = null;

		//templates vector, for not-empty service
		Vector templates = null;
		Vector newtemplates = null;
		Vector<BusinessService> serviceContainer = new Vector<BusinessService>();

		ServiceDetail serviceDetail = null;
		String serviceKey = null;

		int succeededtest = 0;

		//num services to save
		int num_service = services.size();

		//error/success handling
		Result result = null;
		ErrInfo errinfo = null;
		Vector<Result> resultVector = new Vector<Result>();

		try {
			for(int i = 0; i < services.size(); i++)
			{
				serviceut = (BusinessService) services.elementAt(i);	

				//create a Result object and ErrInfo for the binding template under test
				result = new Result();
				result.setKeyType(uddi_org.api_v2.KeyType.SERVICE_KEY.toString());
				errinfo = new ErrInfo();

				//testing an empty service, is always ok
				if (serviceut.getBindingTemplates().getBindingTemplateVector().size() == 0)
				{  
					logger.info("businessServiceTesting: no bt in this service ");

					//prepare the service to save
					serviceContainer.removeAllElements();
					serviceContainer.add(serviceut);

					//no binding template to save, this is en empty service, is always ok
					serviceDetail = this.proxy.save_service(authInfo, serviceContainer);

					this.serviceDetailVector.add((BusinessService)serviceDetail.getBusinessServiceVector().firstElement());
					serviceKey =  ((BusinessService)(serviceDetail.getBusinessServiceVector().firstElement())).getServiceKey();
					logger.info("empty saved service "+ serviceut.getServiceKey());
					succeededtest++;

					//set result object
					result.setErrno(errno_success);
					errinfo.setText(serviceKey);
					errinfo.setErrCode(DispositionReport.E_success);
				}

				else 
				{
					//get templates vector
					templates = serviceut.getBindingTemplates().getBindingTemplateVector();
					logger.info("businessServiceTesting: bt in this service "+ templates.size());

					//save en empty service: a service key is generated
					//or get the service key if this is an update
					if(serviceut.getServiceKey() == "")
					{
						serviceContainer.removeAllElements();
						serviceut.setBindingTemplates(null);
						serviceContainer.add(serviceut);
						serviceDetail = this.proxy.save_service(authInfo, serviceContainer);

						//get the generated serviceKey
						serviceKey = ((BusinessService)(serviceDetail.getBusinessServiceVector().firstElement())).getServiceKey();
						logger.info("businessService test: generated empty service key to fill "+serviceKey);
					}
					else 
					{
						serviceKey = serviceut.getServiceKey();
						logger.info("businessService test: service key to update "+serviceKey);
					}

					//set the generated service key in bindingtemplates to save
					newtemplates = setServiceKey(templates, serviceKey);

					//save and test 
					bindingTemplateTesting(authInfo, newtemplates);

					//clean serviceContainer
					serviceContainer.removeAllElements();

					// set the original BindingTemplates 
					BindingTemplates te = new BindingTemplates();
					te.setBindingTemplateVector(newtemplates); //templates or newtemplates
					serviceut.setBindingTemplates(te);

					//set serviceKey to update
					serviceut.setServiceKey(serviceKey);
					serviceContainer.add(serviceut);

					//if all bt belonging to this service have been successful tested, 
					//the service is updated
					if (this.isAllBindingTestok())
					{
						logger.info("businessService, all binding test ok");
						serviceDetail = this.proxy.save_service(authInfo, serviceContainer);
						this.serviceDetailVector.add((BusinessService)serviceDetail.getBusinessServiceVector().firstElement());
						succeededtest++;

						//set result object
						result.setErrno(errno_success);
						errinfo.setText(serviceKey);
						errinfo.setErrCode(DispositionReport.E_success);
					}

					//the former empty service, and tested bt are deleted (automatically cascade - no proprio)
					else
					{
						logger.info("businessService, NOT all binding test ok, delte service and related bt");

						//delete service
						this.proxy.delete_service(authInfo, serviceKey);

						//delete bt
						deleteRelatedBindingTemplate(authInfo, newtemplates);

						//set result object
						result.setErrno(errno_testFaild);
						errinfo.setErrCode(AuditionManager.E_testFaild);
						errinfo.setText(errno_testFaild_description);
					}
				}

				/*
				 * set errinfo in Result, add the Result object to vector
				 * Results objects in resultVector are in the same order as binding templates has
				 * been processed 
				 */
				result.setErrInfo(errinfo);
				resultVector.add(result);
			}

			this.serviceDispositionReport.setResultVector(resultVector);

			/*
			 * if services are empty or have passed testing
			 * 
			 * this is the only case the client will get a ServiceDetail as a respone to 
			 * save_service api
			 */		
			if(succeededtest == num_service)
			{
				logger.info("all business service tests ok");
				this.allServiceTestok = true;
			}
			else
			{
				logger.info("all business service failed");
				this.allServiceTestok = false;
			}

		}catch (UDDIException e) {
			//throw a new DispositionReportException
			DispositionReportException dre = Util_TypeMapping.checkUDDIException(e);
			throw dre;	} 
		catch(DispositionReportException dre) {
			logger.info("businessServicetestingDRE");
			logger.info(dre.getMessage());	logger.info(dre.toString());
		}
		catch(Exception e ) {
			logger.info("Generic Exception in Audition Manager, testingService");
			logger.info(e.getMessage());
			this.pendingbt.clear();
			this.pendingt.clear();
		}
	}

	public void businessEntityTesting(String authInfo, Vector entities) 
	throws DispositionReportException, TransportException
	{
		logger.info("audition man: businessEntityTest");

		//BusinessEntity under test
		BusinessEntity businessut =  null;

		//
		int num_entity = entities.size();
		int succeededtest = 0;

		//
		Vector<BusinessEntity> businessContainer = new Vector<BusinessEntity>();
		BusinessDetail businessDetail= null;
		Vector services = null;

		String businessKey = null;

		//error/success handling
		Result result = null;
		ErrInfo errinfo = null;
		Vector<Result> resultVector = new Vector<Result>();
		try {
			for(int i = 0; i<entities.size(); i++)
			{
				logger.info("entity size: "+entities.size());
				businessut = (BusinessEntity)entities.elementAt(i);

				//create a Result object and ErrInfo for the binding template under test
				result = new Result();
				result.setKeyType(uddi_org.api_v2.KeyType.BUSINESS_KEY.toString());
				errinfo = new ErrInfo();

				//get the businesskey
				businessKey = businessut.getBusinessKey();

				//case empty business entity
				if(businessut.getBusinessServices().getBusinessServiceVector().size() == 0)
				{
					logger.info("businessEntity testing, no services to test for this entity");
					//prepare the business entity to save
					businessContainer.removeAllElements();
					businessContainer.add(businessut);

					//no service to save, this is en empty business entity, is always ok
					businessDetail = this.proxy.save_business(authInfo, businessContainer);

					this.businessDetailVector.add((BusinessEntity)businessDetail.getBusinessEntityVector().firstElement());
					businessKey =  ((BusinessEntity)(businessDetail.getBusinessEntityVector().firstElement())).getBusinessKey();
					logger.info("saved empty business entity: "+ businessKey);

					succeededtest++;

					//set result object
					result.setErrno(errno_success);
					errinfo.setText(businessKey);
					errinfo.setErrCode(DispositionReport.E_success);

				}
				//the business entity has services inside 
				else
				{
					logger.info("businessEntity testing, one or more services to test for this entity");

					//get templates vector
					services = businessut.getBusinessServices().getBusinessServiceVector();

					//if it is a new operation
					if(businessKey == "")
					{
						//save en empty business: a business key is generated
						businessContainer.removeAllElements();

						businessut.setBusinessServices(null);
						businessContainer.add(businessut);
						businessDetail = this.proxy.save_business(authInfo, businessContainer);

						//get the generated  businessKey
						businessKey = ((BusinessEntity)(businessDetail.getBusinessEntityVector().firstElement())).getBusinessKey();

						logger.info("generated empty business entity to fill: "+businessKey);
					}
					//this is an update operation
					else
					{
						businessKey = businessut.getBusinessKey();
						logger.info(" empty business entity to update: "+businessKey);
					}

					//set the generated business key in services to save
					setBusinessKey(services, businessKey);

					//save and test 
					businessServiceTesting(authInfo, services);

					//clean serviceContainer
					businessContainer.removeAllElements();

					// set the original BusinessServices 
					BusinessServices bs = new BusinessServices();
					bs.setBusinessServiceVector(services);
					businessut.setBusinessServices(bs);

					//set businessKey to update
					businessut.setBusinessKey(businessKey);
					businessContainer.add(businessut);


					/*
					 *if all services belonging to this business entity have been successful tested,
					 *the entity is updated
					 */
					if (this.isAllServiceTestok())
					{
						logger.info("audition manager, business entity, all service test ok");
						businessDetail = this.proxy.save_business(authInfo, businessContainer);
						this.businessDetailVector.add((BusinessEntity)businessDetail.getBusinessEntityVector().firstElement());
						succeededtest++;

						//set result object
						result.setErrno(errno_success);
						errinfo.setText(businessKey);
						errinfo.setErrCode(DispositionReport.E_success);
					}

					/*
					 * The former empty business entity is deleted
					 * Related businessService and bindingTemplate are deleted by caller methods
					 */ 
					else
					{
						logger.info("audition man, business entity, NOT all service test ok");

						this.proxy.delete_business(authInfo, businessKey);

						//set result object
						result.setErrno(errno_testFaild);
						errinfo.setErrCode(AuditionManager.E_testFaild);
						errinfo.setText(errno_testFaild_description);

					}
				}
				result.setErrInfo(errinfo);
				resultVector.add(result);
			}

			this.businessDispositionReport.setResultVector(resultVector);

			/*
			 * if services are empty or have passed testing
			 * 
			 * this is the only case the client will get a ServiceDetail as a respone to 
			 * save_service api
			 */
			if(succeededtest == num_entity)
			{this.allBusinessTestok = true;logger.info("buss entity test: all succeededtest ");}
			else 
			{this.allBusinessTestok = false;logger.info("buss entity test:  NOT all succeededtest ");}

		}catch (UDDIException e) {
			//throw a new DispositionReportException
			DispositionReportException dre = Util_TypeMapping.checkUDDIException(e);
			throw dre;	} 
		catch(DispositionReportException dre) {
			logger.info("businessEntityTesting, businessServiceTesting throws DRE ");
		}
		catch(Exception e ) {
			logger.info("Generic Exception in Audition Manager, testingBusinessEntity");
			logger.info(e.getMessage());
			this.pendingbt.clear();
			this.pendingt.clear();
		}
	}


	/*
	 * tmodel utility methods
	 */

	/**
	 * @param models vector of tModels to check
	 * 
	 * This method checks if one or more of the passed tmodels are "implementation" tmodel
	 * to be marked. This method is called within the save_tModel api. 
	 */
	public void marktModelAsPending(Vector models)
	{
		TModel model = null;
		String tmodelkey = null;
		for(int i =0; i<models.size(); i++)
		{
			model = (TModel) models.elementAt(i);
			tmodelkey = model.getTModelKey();
			boolean b = 
				AuditionManager.istModelBinding(model.getCategoryBag().getKeyedReferenceVector());

			boolean s =
				AuditionManager.istModelService(model.getCategoryBag().getKeyedReferenceVector());

			if((b == true)||(s==true)) 
			{
				this.pendingt.addTModel(new TModelKey(tmodelkey));
				logger.info("added tmodel to pending");
			}
		}
	}

	/**
	 * @param tmodel a vector of tmodels to filter
	 * @return
	 * 
	 * Filters the retrieved tmodels in order to "hide" pending ones
	 * This method is called within the find_tModel api
	 */
	public Vector<TModelInfo> filterTModel(Vector<TModelInfo> tmodel)
	{
		Vector<TModelInfo> s = new Vector<TModelInfo>();

		TModelInfo info = null;
		TModelKey tkey = null;
		logger.info(this.pendingt.toString());

		for(int i = 0; i<tmodel.size(); i++)
		{
			info =  tmodel.elementAt(i);
			tkey = new TModelKey(info.getTModelKey());
			if ((this.pendingt.contains(tkey)) == false)
			{
				s.add(info); 
				logger.info("filterTModel: added "+  tkey.getTmodelkey());
			}
			else
			{
				logger.info("filterTModel: NOT added "+  tkey.getTmodelkey());
			}
		}

		return s;
	}

	/*
	 * utility methods
	 */

	public void deleteRelatedBindingTemplate(String authInfo, Vector t) 
	{
		BindingTemplate template = null;

		for(int i =0; i<t.size(); i++)
		{
			template = (BindingTemplate) t.elementAt(i);
			try {
				this.proxy.delete_binding(authInfo, template.getBindingKey());
			} catch (UDDIException e) {
				logger.info("delete related binding: UDDI exception");
			} catch (TransportException e) {
				logger.info("delete related binding genric exception");
			}
		}

	}

	/**
	 * @param templates
	 * @param serviceKey
	 * @return
	 * 
	 * Use this to update the serviceKey field in a vector of bindingTemplates
	 */
	public Vector setServiceKey(Vector templates, String serviceKey) 
	{
		Vector t = (Vector)templates.clone();
		for(int i =0; i<t.size(); i++)
		{
			((BindingTemplate) t.elementAt(i)).setServiceKey(serviceKey);
		}
		return t;
	}

	/**
	 * @param templates
	 * @param businessKey
	 * 
	 * Use this to update the businessKey field in a vector of businessServices
	 */
	public void setBusinessKey(Vector services, String businessKey) 
	{
		for(int i =0; i<services.size(); i++)
		{
			((BusinessService) services.elementAt(i)).setBusinessKey(businessKey);
		}
	}

	/**
	 * @param v a vector of the KeyedReference for the tmodel 
	 * @return 
	 * <ul>
	 * <li> 1 for the portType tmodel
	 * <li> 2 for the binding tmodel
	 * <li> 3 for the service tmodel
	 * <li> 4 for the SSM tmodel
	 * <ul>
	 * 
	 * find out what kind of tModel it is
	 */
	public static int switchtModelType(Vector v) 
	{
		KeyedReference k = null;
		int result = 0;
		for(int i =0; i <v.size(); i++)
		{
			k = (KeyedReference) v.elementAt(i);

			if((k.getKeyValue().equals(Constants.PORT_TYPE)) && k.getTModelKey().equals(Constants.WSDL_ENTITY_TYPE_TMODELKEY))
				result = 1;

			if((k.getKeyValue().equals(Constants.BINDING)) && k.getTModelKey().equals(Constants.WSDL_ENTITY_TYPE_TMODELKEY))
				result = 2;

			if((k.getKeyValue().equals(Constants.SERVICE)) && k.getTModelKey().equals(Constants.WSDL_ENTITY_TYPE_TMODELKEY))
				result = 3;

			if((k.getKeyValue().equals(Constants.SSM)) && k.getTModelKey().equals(Constants.SSM_SPECIFICATION_TMODELKEY))
				result = 4;
		}
		return result;
	}


	public static boolean istModelBinding(Vector keyedReferenceVector) 
	{			
		return find_keyvalue(keyedReferenceVector, Constants.BINDING, Constants.WSDL_ENTITY_TYPE_TMODELKEY);
	}

	public static boolean istModelService(Vector keyedReferenceVector) 
	{			
		return find_keyvalue(keyedReferenceVector, Constants.SERVICE, Constants.WSDL_ENTITY_TYPE_TMODELKEY);
	}

	/**
	 * checks if the specified keyvalue is in the the keyedReference vector
	 */
	public static boolean find_keyvalue(Vector v, String keyvalue, String tmodelkey) 
	{
		boolean key = false;
		KeyedReference k = null;
		for(int i =0; i <v.size(); i++)
		{
			k = (KeyedReference) v.elementAt(i);
			if((k.getKeyValue().equals(keyvalue)) && k.getTModelKey().equals(tmodelkey))
				key = true;
		}
		return key;
	}

	public static String getAccessPoint(AccessPoint ap)
	{
		String accesspoint = null;
		/*
		 * TODO: in case of other types of URLType: mailto, ftp, fax, phone, other
		 * just ignore this
		 */
		String urltype = ap.getURLType();
		//if ( (urltype.equals("http")) ||(urltype.equals("https")))

		if(urltype.equals("other")) {}
		accesspoint = ap.getText();	
		return accesspoint;

	}

	/*
	 * the wsdl:service name attribute is mapped to the canonical tmodel "XML local name tmodel"
	 * This canonical tmodel provides a mechanism to indicate the name attribute for uddi:businessService
	 */
	public static String getServiceLocalName(Vector v)
	{
		String serviceName = null;
		KeyedReference k = null;
		for(int i =0; i <v.size(); i++)
		{
			k = (KeyedReference) v.elementAt(i);
			if(k.getTModelKey().equals(Constants.XML_LOCAL_NAME_TMODELKEY));
			serviceName = k.getKeyValue();
		}
		return serviceName;
	}

	public static String getServiceNamespace(Vector v)
	{
		String namespace = null;
		KeyedReference k = null;
		for(int i =0; i <v.size(); i++)
		{
			k = (KeyedReference) v.elementAt(i);
			if (k.getTModelKey().equals(Constants.XML_NAMESPACE_TMODELKEY))
				namespace = k.getKeyValue();
		}
		return namespace;
	}

	/*
	 * getters and setters
	 */

	public Vector<BindingTemplate> getBindingDetailVector() {
		return bindingDetailVector;
	}

	public void setBindingDetailVector(Vector<BindingTemplate> bindingDetailVector) {
		this.bindingDetailVector = bindingDetailVector;
	}

	public DispositionReport getBindingDispositionReport() {
		return bindingDispositionReport;
	}

	public void setBindingDispositionReport(DispositionReport dispositionReport) {
		this.bindingDispositionReport = dispositionReport;
	}

	public boolean isAllBindingTestok() {
		return allBindingTestok;
	}

	public void setAllBindingTestok(boolean allTestok) {
		this.allBindingTestok = allTestok;
	}

	public boolean isAllServiceTestok() {
		return allServiceTestok;
	}

	public void setAllServiceTestok(boolean allServiceTestok) {
		this.allServiceTestok = allServiceTestok;
	}

	public Vector<BusinessService> getServiceDetailVector() {
		return serviceDetailVector;
	}

	public void setServiceDetailVector(Vector<BusinessService> serviceDetailVector) {
		this.serviceDetailVector = serviceDetailVector;
	}

	public DispositionReport getServiceDispositionReport() {
		return serviceDispositionReport;
	}

	public void setServiceDispositionReport(
			DispositionReport serviceDispositionReport) {
		this.serviceDispositionReport = serviceDispositionReport;
	}

	public boolean isAllBusinessTestok() {
		return allBusinessTestok;
	}

	public void setAllBusinessTestok(boolean allBusinessTestok) {
		this.allBusinessTestok = allBusinessTestok;
	}

	public Vector<BusinessEntity> getBusinessDetailVector() {
		return businessDetailVector;
	}

	public void setBusinessDetailVector(Vector<BusinessEntity> businessDetailVector) {
		this.businessDetailVector = businessDetailVector;
	}

	public DispositionReport getBusinessDispositionReport() {
		return businessDispositionReport;
	}

	public void setBusinessDispositionReport(
			DispositionReport businessDispositionReport) {
		this.businessDispositionReport = businessDispositionReport;
	}

	public PendingBindingTemplate getPendingbt() {
		return pendingbt;
	}

	public void setPendingbt(PendingBindingTemplate pendingbt) {
		this.pendingbt = pendingbt;
	}

	public PendingTModel getPendingt() {
		return pendingt;
	}

	public void setPendingt(PendingTModel pendingt) {
		this.pendingt = pendingt;
	}

}
