package utils;

import java.util.Vector;

import org.uddi4j.datatype.binding.AccessPoint;
import org.uddi4j.datatype.binding.BindingTemplate;
import org.uddi4j.datatype.binding.HostingRedirector;
import org.uddi4j.datatype.binding.TModelInstanceDetails;
import org.uddi4j.response.BindingDetail;

/**
 * @author faith
 *
 * Feb 24, 2007  7:43:47 PM
 *
 */

public class BindingTemplate_TypeMapping {

	public static Vector toVector(uddi_org.api_v2.BindingTemplate[] array)
	{
		Vector v = new Vector();		
		for(int i =0; i<array.length; i++)
		{
			v.add(transformBindingTemplate(array[i]));
		}
		return v;
	}

	public static BindingTemplate transformBindingTemplate(uddi_org.api_v2.BindingTemplate b) 
	{
		BindingTemplate t = new BindingTemplate();

		//setDescriptionVecor
		t.setDescriptionVector(Util_TypeMapping.toVectorD(b.getDescriptionArray()));

		//set choice element
		if(b.isSetHostingRedirector() == true)
			t.setHostingRedirector(new HostingRedirector(b.getHostingRedirector().getBindingKey()));

		if(b.isSetAccessPoint() == true) {
			AccessPoint accesspoint = new AccessPoint();
			accesspoint.setText(b.getAccessPoint().getStringValue());
			accesspoint.setURLType(b.getAccessPoint().getURLType().toString());
			t.setAccessPoint(accesspoint);
		}

		//setServiceKey
		if (b.isSetServiceKey() == true)
			t.setServiceKey(b.getServiceKey());

		//setBindingKey
		t.setBindingKey(b.getBindingKey());

		//set tmodel instance details
		TModelInstanceDetails details = new TModelInstanceDetails();
		details.setTModelInstanceInfoVector(TModel_TypeMapping.toVectorTMInstanceInfo(b.getTModelInstanceDetails().getTModelInstanceInfoArray()));
		t.setTModelInstanceDetails(details);

		return t;
	}

	public static uddi_org.api_v2.BindingDetail transformBindingDetail(BindingDetail d) 
	{
		//in case of empty BindingDetail parameter returns an empty uddi_org.api_v2.BindingDetail
		
		uddi_org.api_v2.BindingDetail b = uddi_org.api_v2.BindingDetail.Factory.newInstance();
		
		//set attributes: generic, operator, truncated
		b.setGeneric(d.GENERIC);
		
		if(d.getOperator() != null)
			b.setOperator(d.getOperator());
		
		if (d.getTruncated() != null) {
		if (d.getTruncatedBoolean() == true)
			b.setTruncated(uddi_org.api_v2.Truncated.TRUE);
		else 
			b.setTruncated(uddi_org.api_v2.Truncated.FALSE);}

		//set bindingTemplateArray
		if(d.getBindingTemplateVector().size() != 0)
			b.setBindingTemplateArray(toArray(d.getBindingTemplateVector()));

		return b;
	}

	public static uddi_org.api_v2.BindingTemplate[] toArray(Vector v)
	{
		uddi_org.api_v2.BindingTemplate[] array = new uddi_org.api_v2.BindingTemplate[v.size()];
		uddi_org.api_v2.BindingTemplate template = null;
		BindingTemplate t = null;
		for(int i=0; i<v.size(); i++)
		{
			t = (BindingTemplate) v.elementAt(i);
			template =transformBindingTemplate(t);

			array[i] = template;
		}

		return array;
	}

	public static uddi_org.api_v2.BindingTemplate transformBindingTemplate(BindingTemplate t)
	{
		uddi_org.api_v2.BindingTemplate b = uddi_org.api_v2.BindingTemplate.Factory.newInstance();

		//set serviceKey, bindingkey
		b.setServiceKey(t.getServiceKey());
		b.setBindingKey(t.getBindingKey());

		//set description
		b.setDescriptionArray(Util_TypeMapping.toArrayD(t.getDescriptionVector()));

		//set choice element hostingRed or AccessPoint
		uddi_org.api_v2.HostingRedirector hr = uddi_org.api_v2.HostingRedirector.Factory.newInstance();
		if (t.getHostingRedirector()!=null) {
			hr.setBindingKey(t.getHostingRedirector().getBindingKey());
			b.setHostingRedirector(hr);
		}

		uddi_org.api_v2.AccessPoint ap = uddi_org.api_v2.AccessPoint.Factory.newInstance();
		if (t.getAccessPoint()!=null) {
			ap.setStringValue(t.getAccessPoint().getText());
			ap.setURLType(enumURLType(t.getAccessPoint().getURLType()));
			b.setAccessPoint(ap);
		}

		//set tmodelInstanceDetails
		uddi_org.api_v2.TModelInstanceDetails tmid = uddi_org.api_v2.TModelInstanceDetails.Factory.newInstance();
		uddi_org.api_v2.TModelInstanceInfo[] tmiiarray = TModel_TypeMapping.toArrayTModelInstanceInfo(t.getTModelInstanceDetails().getTModelInstanceInfoVector());
		tmid.setTModelInstanceInfoArray(tmiiarray);
		b.setTModelInstanceDetails(tmid);
	
		
		
		return b;
	}

	/**
	 * @param s
	 * @return
	 */
	private static uddi_org.api_v2.URLType.Enum enumURLType (String s)
	{
		if (s.equals("mailto"))
			return uddi_org.api_v2.URLType.MAILTO;
		if (s.equals("http"))
			return uddi_org.api_v2.URLType.HTTP;	
		if (s.equals("https"))
			return uddi_org.api_v2.URLType.HTTPS;
		if (s.equals("ftp"))
			return uddi_org.api_v2.URLType.FTP;
		if (s.equals("fax"))
			return uddi_org.api_v2.URLType.FAX;
		if (s.equals("phone"))
			return uddi_org.api_v2.URLType.PHONE;
		if (s.equals("other"))
			return uddi_org.api_v2.URLType.OTHER;

		return uddi_org.api_v2.URLType.OTHER;
	}

}
