package utils;

import java.util.Vector;

import org.uddi4j.datatype.business.BusinessEntity;
import org.uddi4j.datatype.business.Contacts;
import org.uddi4j.datatype.service.BusinessServices;
import org.uddi4j.response.BusinessDetail;
import org.uddi4j.response.BusinessDetailExt;
import org.uddi4j.response.BusinessEntityExt;
import org.uddi4j.response.BusinessInfo;
import org.uddi4j.response.BusinessList;
import org.uddi4j.response.RelatedBusinessInfo;
import org.uddi4j.response.RelatedBusinessesList;
import org.uddi4j.response.SharedRelationships;
import org.uddi4j.util.CategoryBag;
import org.uddi4j.util.DiscoveryURL;
import org.uddi4j.util.DiscoveryURLs;
import org.uddi4j.util.IdentifierBag;


/**
 * @author faith
 *
 * Feb 28, 2007  1:07:46 AM
 *
 */

public class BusinessEntity_TypeMapping {

	public static Vector toVector(uddi_org.api_v2.BusinessEntity[] array) 
	{
		Vector v = new Vector();
		for(int i =0; i<array.length; i++)
		{
			v.add(transformBusinessEntity(array[i]));
		}

		return v;
	}

	public static BusinessEntity transformBusinessEntity(uddi_org.api_v2.BusinessEntity entity)
	{
		BusinessEntity b = new BusinessEntity();

		//set attributes
		b.setAuthorizedName(entity.getAuthorizedName());
		b.setBusinessKey(entity.getBusinessKey());
		b.setOperator(entity.getOperator());

		//set discoveryURLs
		DiscoveryURLs disc = new DiscoveryURLs();
		Vector v = new Vector();
		if (entity.getDiscoveryURLs() != null)
			v = toVector(entity.getDiscoveryURLs().getDiscoveryURLArray());
		
		disc.setDiscoveryURLVector(v);
		b.setDiscoveryURLs(disc);

		//set name
		v = new Vector();
		if(entity.getNameArray() != null)
			v = Util_TypeMapping.toVectorN(entity.getNameArray());
		b.setNameVector(v);

		//set description
		v = new Vector();
		if(entity.getDescriptionArray() != null)
			v = Util_TypeMapping.toVectorD(entity.getDescriptionArray());
		b.setDescriptionVector(v);

		//set contacts
		Contacts s =new Contacts();
		v = new Vector();
		if(entity.getContacts() != null)
			v = Util_TypeMapping.toVectorC(entity.getContacts().getContactArray());
		
		s.setContactVector(v);
		b.setContacts(s);

		//set businessServices
		BusinessServices services = new BusinessServices();
		v = new Vector();
		if(entity.getBusinessServices() != null)
			v = BusinessService_TypeMapping.toVectorBusinessService(entity.getBusinessServices().getBusinessServiceArray());
		services.setBusinessServiceVector(v);
		b.setBusinessServices(services);

		//set identifierBag
		IdentifierBag iden = new IdentifierBag();
		v = new Vector();
		if(entity.getIdentifierBag() != null)
			v = Util_TypeMapping.toVectorKeyedReference(entity.getIdentifierBag().getKeyedReferenceArray());
		iden.setKeyedReferenceVector(v);
		b.setIdentifierBag(iden);

		//set categoryBag
		CategoryBag cat = new CategoryBag();
		v = new Vector();
		if(entity.getCategoryBag() != null)
			v = Util_TypeMapping.toVectorKeyedReference(entity.getCategoryBag().getKeyedReferenceArray());
		cat.setKeyedReferenceVector(v);
		b.setCategoryBag(cat);

		return b;
	}

	public static Vector toVector(uddi_org.api_v2.DiscoveryURL[] array) 
	{
		Vector v = new Vector();
		for(int i =0; i<array.length; i++)
		{
			v.add(new DiscoveryURL(array[i].getStringValue(), array[i].getUseType()));
		}
		return v;
	}



	/**
	 * @param d
	 * @return
	 * 
	 * response object
	 * 
	 */
	public static uddi_org.api_v2.BusinessDetail transformBusinessDetail(BusinessDetail d) 
	{
		uddi_org.api_v2.BusinessDetail detail = uddi_org.api_v2.BusinessDetail.Factory.newInstance();

		//set generic, operator, truncated
		detail.setGeneric(d.GENERIC);
		detail.setOperator(d.getOperator());
		if (d.getTruncatedBoolean() == true)
			detail.setTruncated(uddi_org.api_v2.Truncated.TRUE);
		else 
			detail.setTruncated(uddi_org.api_v2.Truncated.FALSE);

		//setBusinessEntity
		detail.setBusinessEntityArray(toArray(d.getBusinessEntityVector()));
		return detail;
	}

	public static uddi_org.api_v2.BusinessEntity[] toArray(Vector v)
	{
		uddi_org.api_v2.BusinessEntity[] array = new uddi_org.api_v2.BusinessEntity[v.size()];
		for(int i =0; i<v.size(); i++)
		{
			array[i] = transformBusinessEntity((BusinessEntity) v.elementAt(i));
		}
		return array;
	}

	public static uddi_org.api_v2.BusinessEntity transformBusinessEntity (BusinessEntity b)
	{
		uddi_org.api_v2.BusinessEntity entity = uddi_org.api_v2.BusinessEntity.Factory.newInstance();

		//set attributes businessKey, authorizedName, operator
		entity.setBusinessKey(b.getBusinessKey());
		entity.setOperator(b.getOperator());
		entity.setAuthorizedName(b.getAuthorizedName());

		//set discoveryURLs
		uddi_org.api_v2.DiscoveryURLs urls = uddi_org.api_v2.DiscoveryURLs.Factory.newInstance();
		uddi_org.api_v2.DiscoveryURL[] d = null;
		if (b.getDiscoveryURLs()!= null)
			d = toArrayDiscovertURL(b.getDiscoveryURLs().getDiscoveryURLVector());
		urls.setDiscoveryURLArray(d);
		entity.setDiscoveryURLs(urls);

		//set name
		uddi_org.api_v2.Name[] n = new uddi_org.api_v2.Name[0];
		if (b.getNameVector() != null)
			n = Util_TypeMapping.toArrayN(b.getNameVector());
		entity.setNameArray(n);

		//set description
		uddi_org.api_v2.Description[] dd = new uddi_org.api_v2.Description[0];
		if (b.getDescriptionVector() != null)
			dd = Util_TypeMapping.toArrayD(b.getDescriptionVector());
		entity.setDescriptionArray(dd);	
			
		//set contacts
		uddi_org.api_v2.Contacts contacts = uddi_org.api_v2.Contacts.Factory.newInstance();
		uddi_org.api_v2.Contact[] c = null;
		if (b.getContacts() != null)
			c = Util_TypeMapping.toArrayContact(b.getContacts().getContactVector());
		contacts.setContactArray(c);
		entity.setContacts(contacts);

		//set businessServices
		uddi_org.api_v2.BusinessServices services = null;
		uddi_org.api_v2.BusinessService[] bs = null;
		if (b.getBusinessServices().getBusinessServiceVector().size() != 0) {
			services = uddi_org.api_v2.BusinessServices.Factory.newInstance();
			bs = BusinessService_TypeMapping.toArrayBusinessService(b.getBusinessServices().getBusinessServiceVector());
			services.setBusinessServiceArray(bs);
			entity.setBusinessServices(services);
		}
			

		//set identifierBag
		uddi_org.api_v2.IdentifierBag id = uddi_org.api_v2.IdentifierBag.Factory.newInstance();
		Vector v = null;
		
		if(b.getIdentifierBag() != null) {
			v = b.getIdentifierBag().getKeyedReferenceVector();

			id.setKeyedReferenceArray(Util_TypeMapping.toArrayKeyedReference(v));
			entity.setIdentifierBag(id);
		}

		//set categoryBag
		uddi_org.api_v2.CategoryBag bag = uddi_org.api_v2.CategoryBag.Factory.newInstance();
		v = null;

		if (b.getCategoryBag() != null) {
			v = b.getCategoryBag().getKeyedReferenceVector();

			bag.setKeyedReferenceArray(Util_TypeMapping.toArrayKeyedReference(v));
			entity.setCategoryBag(bag);
		}

		return entity;
	}
	
	public static uddi_org.api_v2.DiscoveryURL[] toArrayDiscovertURL(Vector v)
	{
		uddi_org.api_v2.DiscoveryURL[] array = new uddi_org.api_v2.DiscoveryURL[v.size()];

		uddi_org.api_v2.DiscoveryURL dis = null;

		DiscoveryURL d = null;

		for(int i =0; i<v.size(); i++)
		{
			d = (DiscoveryURL) v.elementAt(i);
			dis = uddi_org.api_v2.DiscoveryURL.Factory.newInstance();
			dis.setStringValue(d.getText());
			dis.setUseType(d.getUseType());
			array[i] = dis;
		}
		return array;

	}

	public static DiscoveryURLs transformDiscoveryURLs(uddi_org.api_v2.DiscoveryURLs u)
	{
		DiscoveryURLs d = new DiscoveryURLs();

		d.setDiscoveryURLVector(toVector(u.getDiscoveryURLArray()));

		return d;

	}

	public static uddi_org.api_v2.BusinessDetailExt transformBusinessDetailExt(BusinessDetailExt e) 
	{
		uddi_org.api_v2.BusinessDetailExt detail = uddi_org.api_v2.BusinessDetailExt.Factory.newInstance();

		//set attributes
		detail.setGeneric(e.GENERIC);
		detail.setOperator(e.getOperator());
		if (e.getTruncatedBoolean() == true)
			detail.setTruncated(uddi_org.api_v2.Truncated.TRUE);
		else 
			detail.setTruncated(uddi_org.api_v2.Truncated.FALSE);

		//set businessEntityExt array
		uddi_org.api_v2.BusinessEntityExt entity = null;
		BusinessEntityExt ex = null;
		Vector v = e.getBusinessEntityExtVector();
		uddi_org.api_v2.BusinessEntityExt[] array = new uddi_org.api_v2.BusinessEntityExt[v.size()];

		for (int i =0; i<v.size(); i++)
		{
			ex = (BusinessEntityExt) v.elementAt(i);
			entity = uddi_org.api_v2.BusinessEntityExt.Factory.newInstance();

			entity.setBusinessEntity(
					BusinessEntity_TypeMapping.transformBusinessEntity(ex.getBusinessEntity()));

			//extra content??

			array[i] = entity;
		}


		detail.setBusinessEntityExtArray(array);
		return detail;
	}

	public static uddi_org.api_v2.BusinessList transformBusinessList(BusinessList l) 
	{
		uddi_org.api_v2.BusinessList list = uddi_org.api_v2.BusinessList.Factory.newInstance();

		//set attributes
		list.setGeneric(l.GENERIC);
		list.setOperator(l.getOperator());
		if (l.getTruncatedBoolean() == true)
			list.setTruncated(uddi_org.api_v2.Truncated.TRUE);
		else 
			list.setTruncated(uddi_org.api_v2.Truncated.FALSE);

		//set businessInfos
		uddi_org.api_v2.BusinessInfos infos= list.addNewBusinessInfos();
		uddi_org.api_v2.BusinessInfo[] check = new uddi_org.api_v2.BusinessInfo[0];
		if (l.getBusinessInfos().getBusinessInfoVector().size() != 0)
			check = toArrayBusinessInfo(l.getBusinessInfos().getBusinessInfoVector());
		infos.setBusinessInfoArray(check);	

		list.setBusinessInfos(infos);
		return list;
	}

	public static uddi_org.api_v2.BusinessInfo[] toArrayBusinessInfo(Vector v)
	{
		uddi_org.api_v2.BusinessInfo[] array = new uddi_org.api_v2.BusinessInfo[v.size()];
		for (int i =0; i<v.size(); i++)
		{
			array[i] = transformBusinessInfo((BusinessInfo) v.elementAt(i));
		}
		return array;
	}

	public static uddi_org.api_v2.BusinessInfo transformBusinessInfo(BusinessInfo inf) 
	{
		uddi_org.api_v2.BusinessInfo info = uddi_org.api_v2.BusinessInfo.Factory.newInstance();

		//set businessKey
		info.setBusinessKey(inf.getBusinessKey());

		//set name
		if ((inf.getNameVector())!=null)
			info.setNameArray(Util_TypeMapping.toArrayN(inf.getNameVector()));
		else 
			info.setNameArray(null);

		//set description
		if((inf.getDescriptionVector())!=null)
			info.setDescriptionArray(Util_TypeMapping.toArrayD(inf.getDescriptionVector()));
		else 
			info.setDescriptionArray(null);
		
		//set serviceInfos
		uddi_org.api_v2.ServiceInfos infos = uddi_org.api_v2.ServiceInfos.Factory.newInstance();
		if ((inf.getServiceInfos().getServiceInfoVector())!=null)
			infos.setServiceInfoArray(BusinessService_TypeMapping.toArrayServiceInfo
				(inf.getServiceInfos().getServiceInfoVector()));
		else infos.setServiceInfoArray(null);
		
		info.setServiceInfos(infos);

		return info;
	}

	public static uddi_org.api_v2.RelatedBusinessesList transformRelatedBusinessList(RelatedBusinessesList l) 
	{
		uddi_org.api_v2.RelatedBusinessesList list = uddi_org.api_v2.RelatedBusinessesList.Factory.newInstance();

		//set generic, operator, truncated attributes, businesskey
		list.setGeneric(l.GENERIC);
		list.setOperator(l.getOperator());
		list.setBusinessKey(l.getBusinessKey());
		if (l.getTruncatedBoolean() == true)
			list.setTruncated(uddi_org.api_v2.Truncated.TRUE);
		else 
			list.setTruncated(uddi_org.api_v2.Truncated.FALSE);

		//set RelatedBusinessInfo
		uddi_org.api_v2.RelatedBusinessInfos r = uddi_org.api_v2.RelatedBusinessInfos.Factory.newInstance();
		r.setRelatedBusinessInfoArray(toArrayRelatedBusinessInfo(l.getRelatedBusinessInfos().getRelatedBusinessInfoVector()));
		list.setRelatedBusinessInfos(r);

		return list;
	}

	public static uddi_org.api_v2.RelatedBusinessInfo[] toArrayRelatedBusinessInfo (Vector v)
	{
		uddi_org.api_v2.RelatedBusinessInfo[] array = new uddi_org.api_v2.RelatedBusinessInfo[v.size()];
		for (int i =0; i<v.size(); i++)
		{
			array[i] = transformRelatedBusinessInfo((RelatedBusinessInfo) v.elementAt(i));
		}
		return array;
	}

	private static uddi_org.api_v2.RelatedBusinessInfo transformRelatedBusinessInfo(RelatedBusinessInfo i) 
	{
		uddi_org.api_v2.RelatedBusinessInfo info = uddi_org.api_v2.RelatedBusinessInfo.Factory.newInstance();

		info.setBusinessKey(i.getBusinessKey());
		info.setDescriptionArray(Util_TypeMapping.toArrayD(i.getDescriptionVector()));
		info.setNameArray(Util_TypeMapping.toArrayN(i.getNameVector()));
		info.setSharedRelationshipsArray(toArraySharedRelationships(i.getSharedRelationshipsVector()));

		return info;
	}

	public static uddi_org.api_v2.SharedRelationships[] toArraySharedRelationships (Vector v)
	{
		uddi_org.api_v2.SharedRelationships[] array = new uddi_org.api_v2.SharedRelationships[v.size()];
		uddi_org.api_v2.SharedRelationships s = null;
		SharedRelationships sr = null;

		for (int i =0; i<v.size(); i++)
		{
			sr = (SharedRelationships) v.elementAt(i);
			s = uddi_org.api_v2.SharedRelationships.Factory.newInstance();
			s.setKeyedReferenceArray(Util_TypeMapping.toArrayKeyedReference(sr.getKeyedReferenceVector()));
			s.setDirection(enumUtil(sr.getDirection()));

			array[i] = s;
		}
		return array;
	}

	public static uddi_org.api_v2.Direction.Enum enumUtil(String s)
	{
		uddi_org.api_v2.Direction.Enum m = null;

		if (s.equals("fromKey"))
			m = uddi_org.api_v2.Direction.FROM_KEY;

		if (s.equals("toKey"))
			m = uddi_org.api_v2.Direction.TO_KEY;

		return m;
	}

}
