package utils;

import java.util.Vector;
import org.uddi4j.datatype.assertion.PublisherAssertion;
import org.uddi4j.response.AssertionStatusReport;
import org.uddi4j.response.AssertionStatusItem;
import org.uddi4j.response.BusinessInfo;
import org.uddi4j.response.PublisherAssertions;
import org.uddi4j.response.RegisteredInfo;
import org.uddi4j.response.ServiceInfo;
import org.uddi4j.response.TModelInfo;
import org.uddi4j.util.FromKey;
import org.uddi4j.util.KeyedReference;
import org.uddi4j.util.ToKey;



/**
 * @author faith
 *
 * Feb 24, 2007  6:51:40 PM
 *
 */
public class PublisherAssertions_TypeMapping {

	public static Vector toVector(uddi_org.api_v2.PublisherAssertion[] parray) 
	{
		Vector ass = new Vector();
		for(int i =0; i<parray.length; i++)
		{
			ass.add(transformPublisherAssertion(parray[i]));
		}
		return ass;
	}

	public static PublisherAssertion transformPublisherAssertion(uddi_org.api_v2.PublisherAssertion a) 
	{
		PublisherAssertion assertion = new PublisherAssertion();
		
		//setFromKey
		FromKey fromkey = new FromKey(a.getFromKey());
		assertion.setFromKey(fromkey);
		assertion.setFromKeyString(a.getFromKey());
		
		//setToKey
		ToKey tokey = new ToKey(a.getToKey());
		assertion.setToKey(tokey);
		assertion.setToKeyString(a.getToKey());
		
		//setKeyedReference
		KeyedReference keyedreference = new KeyedReference();
		keyedreference.setKeyName(a.getKeyedReference().getKeyName());
		keyedreference.setKeyValue(a.getKeyedReference().getKeyValue());
		keyedreference.setTModelKey(a.getKeyedReference().getTModelKey());
		assertion.setKeyedReference(keyedreference);
		
		return assertion;
	}

	public static uddi_org.api_v2.AssertionStatusReport transformAssertionStatusReport(AssertionStatusReport d) 
	{
		uddi_org.api_v2.AssertionStatusReport a = uddi_org.api_v2.AssertionStatusReport.Factory.newInstance();
		a.setGeneric(d.GENERIC);
		a.setOperator(d.getOperator());
		a.setAssertionStatusItemArray(toArray(d.getAssertionStatusItemVector()));
		
		return a;
	}

	private static uddi_org.api_v2.AssertionStatusItem[] toArray(Vector v) 
	{
		 uddi_org.api_v2.AssertionStatusItem[] array = new uddi_org.api_v2.AssertionStatusItem[v.size()];
		 
		 for(int i=0; i<v.size();i++)
		 {
			 array[i] = transformAssertionStatusItem((AssertionStatusItem) v.elementAt(i));
		 }
		 
		 return array;
	}

	private static uddi_org.api_v2.AssertionStatusItem transformAssertionStatusItem(AssertionStatusItem itema) 
	{
		uddi_org.api_v2.AssertionStatusItem item = uddi_org.api_v2.AssertionStatusItem.Factory.newInstance();
		
		//setCompletionStatus
		 item.setCompletionStatus(itema.getCompletionStatus().getText());
		 
		 //setFromKey
		 item.setFromKey(itema.getFromKeyString());
		 
		 //setToKey
		 item.setToKey(itema.getToKeyString());
		 
		 //setKeyedReference
		 uddi_org.api_v2.KeyedReference keyedreference = uddi_org.api_v2.KeyedReference.Factory.newInstance();
		 keyedreference.setKeyName(itema.getKeyedReference().getKeyName());
		 keyedreference.setKeyValue(itema.getKeyedReference().getKeyValue());
		 item.setKeyedReference(keyedreference);
		 
		 //setKeysOwned
		 uddi_org.api_v2.KeysOwned keysowned = uddi_org.api_v2.KeysOwned.Factory.newInstance();
		 keysowned.setFromKey(itema.getKeysOwned().getFromKeyString());
		 keysowned.setToKey(itema.getKeysOwned().getToKeyString());
		 item.setKeysOwned(keysowned);
		
		return item;
	}

	public static uddi_org.api_v2.PublisherAssertions transformPublisherAssertions(PublisherAssertions d) 
	{
		uddi_org.api_v2.PublisherAssertions s = uddi_org.api_v2.PublisherAssertions.Factory.newInstance();
		
		//setAuthorizedName
		s.setAuthorizedName(d.getAuthorizedName());
		
		//setOperator
		s.setOperator(d.getOperator());
		
		//setGeneric
		s.setGeneric(d.GENERIC);
		
		//setPublisherAssertionArray
		Vector v = d.getPublisherAssertionVector();
		uddi_org.api_v2.PublisherAssertion[] array = new uddi_org.api_v2.PublisherAssertion[v.size()];
		for(int i =0; i<v.size();i++)
		{
			array[i] = transformPublisherAssertion((PublisherAssertion)v.elementAt(i));
		}
		s.setPublisherAssertionArray(array);
	
		return s;
	}

	private static uddi_org.api_v2.PublisherAssertion transformPublisherAssertion(PublisherAssertion a) 
	{
		uddi_org.api_v2.PublisherAssertion ass = uddi_org.api_v2.PublisherAssertion.Factory.newInstance();
		
		ass.setFromKey(a.getFromKeyString());
		
		ass.setToKey(a.getToKeyString());
		
		uddi_org.api_v2.KeyedReference keyedReference = uddi_org.api_v2.KeyedReference.Factory.newInstance();
		keyedReference.setKeyName(a.getKeyedReference().getKeyName());
		keyedReference.setKeyValue(a.getKeyedReference().getKeyValue());
		ass.setKeyedReference(keyedReference);

		return ass;
	}

	
	/**
	 * @param RegisteredInfo r
	 * @return uddi_org.api_v2.RegisteredInfo
	 * 
	 * Map a uddi4j response object into a uddi_org.api_v2 response message
	 * 
	 */
	public static uddi_org.api_v2.RegisteredInfo transformTo(RegisteredInfo r) 
	{
		uddi_org.api_v2.RegisteredInfo info = uddi_org.api_v2.RegisteredInfo.Factory.newInstance();
		
		info.setGeneric(r.GENERIC);
		info.setOperator(r.getOperator());
		if (r.getTruncatedBoolean() == true)
			info.setTruncated(uddi_org.api_v2.Truncated.TRUE);
		else 
			info.setTruncated(uddi_org.api_v2.Truncated.FALSE);
		
		//setBusinessInfo
		Vector v = r.getBusinessInfos().getBusinessInfoVector();
		uddi_org.api_v2.BusinessInfos infos = uddi_org.api_v2.BusinessInfos.Factory.newInstance();
		uddi_org.api_v2.BusinessInfo[] array = new uddi_org.api_v2.BusinessInfo[v.size()];
		uddi_org.api_v2.BusinessInfo inf = null;
		BusinessInfo in = null;
		
		for(int i=0;i<v.size();i++)
		{
			in = (BusinessInfo)v.elementAt(i);
			
			//setBusinessKey
			inf.setBusinessKey(in.getBusinessKey());
			
			//setDescriptionArray
			inf.setDescriptionArray(Util_TypeMapping.toArrayD(in.getDescriptionVector()));
			
			//setNameArray
			inf.setNameArray(Util_TypeMapping.toArrayN(in.getNameVector()));
		
			//setServiceInfos
			uddi_org.api_v2.ServiceInfos serviceinfos =  null;
			uddi_org.api_v2.ServiceInfo[] sericeinfoarray = toArrayS( in.getServiceInfos().getServiceInfoVector());  
			serviceinfos.setServiceInfoArray(sericeinfoarray);
			inf.setServiceInfos(serviceinfos);
			
			array[i] = inf;
		}
		
		infos.setBusinessInfoArray(array);
		info.setBusinessInfos(infos);
		
		//setTModelInfos
		uddi_org.api_v2.TModelInfo[] tmi = toArrayT(r.getTModelInfos().getTModelInfoVector());
		uddi_org.api_v2.TModelInfos infoss = uddi_org.api_v2.TModelInfos.Factory.newInstance();
		infoss.setTModelInfoArray(tmi);
		info.setTModelInfos(infoss);
		
		return info;
	}

	public static uddi_org.api_v2.TModelInfo[] toArrayT(Vector v)
	{
		uddi_org.api_v2.TModelInfo[] infoarray = new uddi_org.api_v2.TModelInfo[v.size()];
		uddi_org.api_v2.TModelInfo info = null; 
		TModelInfo inf = null;
		for(int i =0; i<v.size();i++)
		{
			inf = (TModelInfo) v.elementAt(i);
			info = uddi_org.api_v2.TModelInfo.Factory.newInstance();
			
			//setTModelKey
			info.setTModelKey(inf.getTModelKey());
			
			//setName
			info.setName(Util_TypeMapping.transformName(inf.getName()));
			
			infoarray[i] = info;
		}
		return infoarray;
	}

	public static uddi_org.api_v2.ServiceInfo[] toArrayS(Vector v)
	{
		uddi_org.api_v2.ServiceInfo[] infoarray = new uddi_org.api_v2.ServiceInfo[v.size()];
		uddi_org.api_v2.ServiceInfo info = null;
		ServiceInfo inf = null;
		for(int i =0; i<v.size();i++)
		{
			inf = (ServiceInfo) v.elementAt(i);
			info = uddi_org.api_v2.ServiceInfo.Factory.newInstance();
			
			//setServiceKey
			info.setServiceKey(info.getServiceKey());
			
			//setBusinessKey
			info.setBusinessKey(inf.getBusinessKey());
			
			//setName
			info.setNameArray(Util_TypeMapping.toArrayN(inf.getNameVector()));
			
			infoarray[i]=info;
		}
		return infoarray;
	}

}
