package utils;

import java.util.Vector;

import org.uddi4j.UDDIException;
import org.uddi4j.datatype.Description;
import org.uddi4j.datatype.Name;
import org.uddi4j.datatype.OverviewDoc;
import org.uddi4j.datatype.business.Address;
import org.uddi4j.datatype.business.AddressLine;
import org.uddi4j.datatype.business.Contact;
import org.uddi4j.datatype.business.Email;
import org.uddi4j.datatype.business.Phone;
import org.uddi4j.response.DispositionReport;
import org.uddi4j.response.Result;
import org.uddi4j.util.CategoryBag;
import org.uddi4j.util.FindQualifier;
import org.uddi4j.util.FindQualifiers;
import org.uddi4j.util.IdentifierBag;
import org.uddi4j.util.KeyedReference;

import uddi_org.api_v2.DispositionReportDocument;
import uddi_org.service.DispositionReportException;

/**
 * @author faith
 *
 * Feb 24, 2007  7:19:32 PM
 *
 */
public class Util_TypeMapping {

	public static DispositionReportException checkUDDIException(UDDIException e)
	{
		//get the uddi4j exception
		DispositionReport dr = e.getDispositionReport();

		if(dr != null) 
			return  buildDispositionReportException(dr);
		
		else
			return new DispositionReportException("generic SOAP fault: "+e.getFaultString(), e);
	}

	public static uddi_org.api_v2.DispositionReport transformDispositionReport(DispositionReport d)
	{
		uddi_org.api_v2.DispositionReport disposition = uddi_org.api_v2.DispositionReport.Factory.newInstance();

		//set attributes
		if (d.getGeneric() != null)
			disposition.setGeneric(d.getGeneric());
		disposition.setOperator(d.getOperator());
		if (d.getTruncated() == true)
			disposition.setTruncated(uddi_org.api_v2.Truncated.TRUE);
		else 
			disposition.setTruncated(uddi_org.api_v2.Truncated.FALSE);

		Vector v = d.getResultVector();
		uddi_org.api_v2.Result[] resultarray = new uddi_org.api_v2.Result[v.size()];
		uddi_org.api_v2.Result r = null;
		Result re = null;
		for(int i=0;i<v.size();i++)
		{
			r = uddi_org.api_v2.Result.Factory.newInstance();
			re = (Result) v.elementAt(i);

			//setErrno
			r.setErrno(Integer.parseInt(re.getErrno()));

			//setErrInfo
			uddi_org.api_v2.ErrInfo errinfo = uddi_org.api_v2.ErrInfo.Factory.newInstance();
			errinfo.setErrCode(re.getErrInfo().getErrCode());
			errinfo.setStringValue(re.getErrInfo().getText()); /*errore se chimato da remoto*/
			r.setErrInfo(errinfo);

			//setKeyType
			String keytype = re.getKeyType();
			if (keytype.equals("businessKey"))
				r.setKeyType(uddi_org.api_v2.KeyType.BUSINESS_KEY);
			if (keytype.equals("tModelKey"))
				r.setKeyType(uddi_org.api_v2.KeyType.T_MODEL_KEY);
			if (keytype.equals("serviceKey"))
				r.setKeyType(uddi_org.api_v2.KeyType.SERVICE_KEY);
			if (keytype.equals("bindingKey"))
				r.setKeyType(uddi_org.api_v2.KeyType.BINDING_KEY);		

			resultarray[i] = r;
		}
		disposition.setResultArray(resultarray);
		
		return disposition;
	}
	
	public static DispositionReportException buildDispositionReportException(DispositionReport d) 
	{
		//create a new DispositionReportDocument to encapsulate in the DispositionReportException to be throw
		DispositionReportDocument drd= DispositionReportDocument.Factory.newInstance();
		uddi_org.api_v2.DispositionReport drr = drd.addNewDispositionReport();
		drr = Util_TypeMapping.transformDispositionReport(d);
		drd.setDispositionReport(drr);

		//create a new DispositionReportException
		DispositionReportException dre = new DispositionReportException();
		dre.setFaultMessage(drd);
		return dre;
	}

	public static Vector toVector(String[] keyArray) 
	{
		Vector v = new Vector();
		for(int i=0; i<keyArray.length; i++)
		{
			v.add(new String(keyArray[i]));
		}
		return v;
	}

	public static Vector toVectorD(uddi_org.api_v2.Description[] array)
	{
		Vector v = new Vector();

		uddi_org.api_v2.Description d = null;
		Description s = null;

		for(int i=0; i<array.length; i++)
		{
			d = array[i];
			s = new Description();

			s.setLang(d.getLang());
			s.setText(d.getStringValue());

			v.add(s);
		}

		return v;
	}

	public static Vector toVectorN(uddi_org.api_v2.Name[] array)
	{
		Vector v = new Vector();

		uddi_org.api_v2.Name d = null;
		Name s = null;

		for(int i=0; i<array.length; i++)
		{
			d = array[i];
			s = new Name();

			s.setLang(d.getLang());
			s.setText(d.getStringValue());

			v.add(s);
		}


		return v;
	}

	public static uddi_org.api_v2.Description[] toArrayD(Vector v) 
	{
		uddi_org.api_v2.Description[] d = new uddi_org.api_v2.Description[v.size()];

		for(int i=0;i<v.size();i++)
		{
			d[i] = transformDescription((Description)v.elementAt(i));
		}
		return d;
	}

	public static uddi_org.api_v2.Description transformDescription(Description d) 
	{
		uddi_org.api_v2.Description desc = uddi_org.api_v2.Description.Factory.newInstance();

		if (!(d.getLang().equals("")))
			desc.setLang(d.getLang());

		desc.setStringValue(d.getText());

		return desc;
	}

	public static uddi_org.api_v2.Name[] toArrayN(Vector v)
	{
		uddi_org.api_v2.Name[] n = new uddi_org.api_v2.Name[v.size()];
		for(int i=0;i<v.size();i++)
		{
			n[i] = transformName((Name)v.elementAt(i));
		}
		return n;
	}

	/**
	 * @param n
	 * @return
	 * 
	 * TODO: check this bloody lang attribute!!!
	 * in Description as well
	 */
	public static uddi_org.api_v2.Name transformName(Name n)
	{
		uddi_org.api_v2.Name nn = uddi_org.api_v2.Name.Factory.newInstance();

		if (n.getLang() != null)
			if (!(n.getLang().equals("")))
				nn.setLang(n.getLang());

		nn.setStringValue(n.getText());

		return nn;
	}

	public static Name transformName(uddi_org.api_v2.Name n)
	{
		Name name = new Name();

		name.setText(n.getStringValue());
		name.setLang(n.getLang());

		return name;
	}

	public static Vector toVectorC(uddi_org.api_v2.Contact[] array) 
	{
		Vector v = new Vector();

		uddi_org.api_v2.Contact c = null;
		Contact contact = null;

		for(int i=0;i<array.length;i++)
		{
			c = array[i];
			contact = new Contact();

			//setPersonName
			contact.setPersonName(c.getPersonName());

			//setDescription
			contact.setDescriptionVector(toVectorD(c.getDescriptionArray()));

			//setPhone
			contact.setPhoneVector(toVectorPhone(c.getPhoneArray()));

			//setEmail
			contact.setEmailVector(toVectorEmail(c.getEmailArray()));

			//setAddress
			contact.setAddressVector(toVectorAddress(c.getAddressArray()));

			v.add(contact);
		}
		return v;
	}

	public static uddi_org.api_v2.Contact[] toArrayContact (Vector v)
	{
		uddi_org.api_v2.Contact[] array = new uddi_org.api_v2.Contact[v.size()];

		uddi_org.api_v2.Contact contact = null;
		Contact c = null;

		for(int i =0; i<v.size(); i++)
		{
			c = (Contact) v.elementAt(i);
			contact = uddi_org.api_v2.Contact.Factory.newInstance();

			contact.setUseType(c.getUseType());
			contact.setPersonName(c.getPersonNameString());
			contact.setDescriptionArray(toArrayD(c.getDescriptionVector()));
			contact.setPhoneArray(toArrayPhone(c.getPhoneVector()));
			contact.setEmailArray(toArrayEmail(c.getEmailVector()));
			contact.setAddressArray(toArrayAddress(c.getAddressVector()));

			array[i] = contact;
		}

		return array;
	}

	public static uddi_org.api_v2.Address[] toArrayAddress(Vector v)
	{
		uddi_org.api_v2.Address[] array = new uddi_org.api_v2.Address[v.size()];
		uddi_org.api_v2.Address address = null;
		Address a = null;
		for(int i=0;i<v.size();i++)
		{
			a = (Address) v.elementAt(i);
			address = uddi_org.api_v2.Address.Factory.newInstance();

			address.setSortCode(a.getSortCode());
			address.setTModelKey(a.getTModelKey());
			address.setUseType(a.getUseType());
			address.setAddressLineArray(toArrayAddressLine(a.getAddressLineVector()));

			array[i] = address;
		}
		return array;
	} 

	public static uddi_org.api_v2.AddressLine[] toArrayAddressLine(Vector v)
	{
		uddi_org.api_v2.AddressLine[] array = new uddi_org.api_v2.AddressLine[v.size()];
		uddi_org.api_v2.AddressLine line = null;
		AddressLine e = null;
		for(int i=0;i<v.size();i++)
		{
			e = (AddressLine) v.elementAt(i);
			line = uddi_org.api_v2.AddressLine.Factory.newInstance();

			line.setStringValue(e.getText());
			line.setKeyName(e.getKeyName());
			line.setKeyValue(e.getKeyValue());

			array[i] = line;
		}
		return array;
	} 

	public static uddi_org.api_v2.Email[] toArrayEmail(Vector v)
	{
		uddi_org.api_v2.Email[] array = new uddi_org.api_v2.Email[v.size()];
		uddi_org.api_v2.Email email = null;
		Email e = null;
		for(int i=0;i<v.size();i++)
		{
			e = (Email) v.elementAt(i);
			email = uddi_org.api_v2.Email.Factory.newInstance();

			email.setStringValue(e.getText());
			email.setUseType(e.getUseType());

			array[i] = email;
		}
		return array;
	} 

	public static uddi_org.api_v2.Phone[] toArrayPhone(Vector v)
	{
		uddi_org.api_v2.Phone[] array = new uddi_org.api_v2.Phone[v.size()];
		uddi_org.api_v2.Phone phone = null;
		Phone p = null;
		for(int i=0;i<v.size();i++)
		{
			p = (Phone) v.elementAt(i);
			phone = uddi_org.api_v2.Phone.Factory.newInstance();

			phone.setStringValue(p.getText());
			phone.setUseType(p.getUseType());

			array[i] = phone;
		}
		return array;
	}

	public static Vector toVectorAddress(uddi_org.api_v2.Address[] array)
	{
		Vector v = new Vector();
		uddi_org.api_v2.Address address = null;
		Address a = null;

		for(int i=0;i<array.length;i++)
		{
			address = array[i];
			a = new Address();

			a.setUseType(address.getUseType());
			a.setTModelKey(address.getTModelKey());
			a.setSortCode(address.getSortCode());
			a.setAddressLineVector(toVectorAddressLine(address.getAddressLineArray()));

			v.add(a);
		}
		return v;
	}

	private static Vector toVectorAddressLine(uddi_org.api_v2.AddressLine[] array) 
	{
		Vector v = new Vector();
		uddi_org.api_v2.AddressLine line = null;
		AddressLine a = null;

		for(int i=0;i<array.length;i++)
		{
			line = array[i];
			a = new AddressLine();

			a.setText(line.getStringValue());
			a.setKeyName(line.getKeyName());
			a.setKeyValue(line.getKeyValue());

			v.add(a);
		}
		return v;
	}

	private static Vector toVectorEmail(uddi_org.api_v2.Email[] array) 
	{
		Vector v = new Vector();
		uddi_org.api_v2.Email email = null;
		Email e = null;

		for(int i=0;i<array.length;i++)
		{
			email = array[i];
			e = new Email();

			e.setText(email.getStringValue());
			e.setUseType(email.getUseType());
			v.add(e);
		}
		return v;
	}

	private static Vector toVectorPhone(uddi_org.api_v2.Phone[] array) 
	{
		Vector v = new Vector();

		uddi_org.api_v2.Phone phone = null;
		Phone p = null;

		for(int i=0;i<array.length;i++)
		{
			phone = array[i];
			p = new Phone();

			p.setText(phone.getStringValue());
			p.setUseType(phone.getUseType());

			v.add(p);
		}
		return v;
	}

	public static Vector toVectorKeyedReference(uddi_org.api_v2.KeyedReference[] array)
	{
		Vector v = new Vector();

		uddi_org.api_v2.KeyedReference a = null;
		KeyedReference keyedreference = null;

		for(int i=0;i<array.length;i++)
		{
			a = array[i];
			keyedreference = new KeyedReference();

			keyedreference.setKeyName(a.getKeyName());
			keyedreference.setKeyValue(a.getKeyValue());
			keyedreference.setTModelKey(a.getTModelKey());

			v.add(keyedreference);
		}
		return v;
	}

	public static KeyedReference transformKeyedReference(uddi_org.api_v2.KeyedReference r)
	{
		KeyedReference k = new KeyedReference();

		return k;
	}

	public static uddi_org.api_v2.KeyedReference[] toArrayKeyedReference(Vector v)
	{
		if(v == null) 
			return new uddi_org.api_v2.KeyedReference[0];

		uddi_org.api_v2.KeyedReference[] array = new uddi_org.api_v2.KeyedReference[v.size()];


		uddi_org.api_v2.KeyedReference a = null;
		KeyedReference keyedreference = null;

		for(int i=0;i<v.size();i++)
		{
			keyedreference = (KeyedReference) v.elementAt(i);
			a = uddi_org.api_v2.KeyedReference.Factory.newInstance();

			a.setKeyName(keyedreference.getKeyName());
			a.setKeyValue(keyedreference.getKeyValue());
			a.setTModelKey(keyedreference.getTModelKey());

			array[i] = a;
		}

		return array;
	}

	public static OverviewDoc transformOverviwDoc(uddi_org.api_v2.OverviewDoc o) 
	{
		OverviewDoc r = new OverviewDoc();

		r.setDescriptionVector(toVectorD(o.getDescriptionArray()));
		r.setOverviewURL(o.getOverviewURL());

		return r;
	}

	public static uddi_org.api_v2.OverviewDoc transformOverviwDoc(OverviewDoc d)
	{
		uddi_org.api_v2.OverviewDoc o = uddi_org.api_v2.OverviewDoc.Factory.newInstance();

		o.setDescriptionArray(toArrayD(d.getDescriptionVector()));
		o.setOverviewURL(d.getOverviewURLString());

		return o;
	}

	public static FindQualifiers transformFindBinding(uddi_org.api_v2.FindQualifiers f) 
	{
		FindQualifiers q = null;

		Vector v = new Vector();
		if(f != null) {
			q = new FindQualifiers();
			String[] array = f.getFindQualifierArray();

			FindQualifier r = null;

			for(int i =0; i<array.length; i++)
			{
				r = new FindQualifier(array[i]);
				v.add(r);
			}

			q.setFindQualifierVector(v);
		}
		return q;
		
	}

	public static CategoryBag transformCategoryBag(uddi_org.api_v2.CategoryBag c) 
	{
		CategoryBag g = new CategoryBag();
		g.setKeyedReferenceVector(toVectorKeyedReference(c.getKeyedReferenceArray()));
		return g;
	}

	public static IdentifierBag transformIdentifierBag(uddi_org.api_v2.IdentifierBag b) 
	{
		IdentifierBag id = new IdentifierBag();
		id.setKeyedReferenceVector(toVectorKeyedReference(b.getKeyedReferenceArray()));
		return id;
	}

}
